% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: lex_rules.pl,v $
%%  $Revision: 1.5 $
%%      $Date: 2005/12/14 14:06:56 $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Eine kleine Spielzeuggrammatik fr die Lehre
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.10.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile if/2.
:- lex_rule_depth(5).

reorder_acc_dat_lr lex_rule
(head:Head,
 subcat:[NP_nom,NP_dat,NP_acc|Rest],
 reorder:minus,
 Dtr)
**>
(head:Head,
 subcat:[NP_nom,NP_acc,NP_dat|Rest],
 reorder:plus,
 dtr:Dtr)
morphs
  X becomes X.



%%
%% inflection lexical rules
%%

det_infl_lr lex_rule
 (stem,
  head:(Head,
        det),
  infl:affix:(det_i_suffix,
              phon:[(a_ Suff)],
              num:Num,
              gen:Gen,
              dtype:DType,
              case:Case),
  subcat:Subcat,
  Dtr)
**>
 (head:(Head,
        agr:(num:Num,
             gen:Gen,
             dtype:DType,
             case:Case)),
  subcat:Subcat,
  dtr:Dtr)
morphs
  X becomes (X,Suff).

attr_adj_infl_lr lex_rule
(stem,
 head:(Head,
       adj),
 subcat:Subcat,
 infl:affix:(attr_adj_i_suffix,
             phon:[(a_ Suff)],
             dtype:DType,
             num:Num,
             gen:Gen,
             case:Case),
 Dtr)
**>
(head:(Head,
       agr:(num:Num,
            gen:Gen,
            case:Case,
            dtype:DType)),
 subcat:Subcat,
 dtr:Dtr)
morphs
  X becomes (X,Suff).

noun_infl_lr lex_rule
(stem,
 head:(Head,
        noun),
 subcat:Subcat,
 infl:(umlaut:UmlautStem,
       affix:(Affix,
              noun_i_suffix,
              phon:[(a_ Suff)],
              num:Num,
              case:Case)),
 Dtr)
**>
( head:(Head,
        agr:(num:Num,
             case:Case)),
  subcat:Subcat,
  dtr:Dtr)
if
  get_irreg_n_stem(Affix,UmlautStem,(a_ IStem))
morphs
  X becomes (X,Suff) when IStem = none,
  X becomes (IStem,Suff).


verb_infl_lr lex_rule
( stem,
  head:(Head,
        verb),
  subcat:Subcat,
  infl:(v_stems:VStems,
        affix:(Affix,
               fin_verb_i_suffix,
               phon:[(a_ AffixPhon)],
               per:Per,
               num:Num)),
  Dtr) 
 **>
( head:Head,
  subcat:(Subcat,
          [(head:agr:(per:Per,
                      num:Num))|_]),
  dtr:Dtr)
if (get_irreg_v_stem(Affix,VStems,a_ IStem))
morphs
  X becomes (X,AffixPhon) when IStem = none,
  X becomes (IStem,AffixPhon).






get_irreg_v_stem((Affix,
                  phon:Phon),VStems,IrregStem) if
  when(Phon=(e_list;ne_list)
       ,undelayed_get_irreg_v_stem(Affix,VStems,IrregStem)).


undelayed_get_irreg_v_stem((temo:pres_ind_conj),                (pres2:(a_ none)),     (a_ none    ))  if !,true.

% ich fange, du fngst, er fngt
% ich habe, du hast, er hat
undelayed_get_irreg_v_stem((fk:(v_strong;mixed;v_weak),
                            temo:pres_ind, per:second_or_third,num:sg),(pres2:(a_ Pres2Stem)),(a_ Pres2Stem)) if !,true.


undelayed_get_irreg_v_stem((fk:(v_strong;mixed;v_weak),
                            temo:pres_imp),                     (imp_st:(a_ Pres2Stem)),(a_ Pres2Stem)) if !,true.

% Ich darf, du darfst, er darf, wir drfen, ihr drft, sie drfen
% the irregular stem in sg:
undelayed_get_irreg_v_stem((fk:modal,
                            temo:pres_ind,
                            num:sg),                            (pres2:(a_ Pres2Stem)),(a_ Pres2Stem)) if !,true.


undelayed_get_irreg_v_stem((temo:pres_conj,per:second_or_third,num:sg),_,     (a_ none    ))  if !,true.

% ich fange, ich habe
undelayed_get_irreg_v_stem((fk:(v_strong;mixed),
                            temo:pres_ind_conj,per:first,num:sg),      _,                     (a_ none    ))  if !,true.


% versuchtest
undelayed_get_irreg_v_stem((temo:past_ind_conj),                (past:(a_ none),
                                                                 conj:(a_ none)),      (a_ none    ))  if !,true.

% ich fing, ..., ich durfte, ...
undelayed_get_irreg_v_stem((temo:past_ind),                     (past:(a_ PastStem)),  (a_ PastStem))  if !,true.

% If a conj stem is definied, take it, otherwise return the past stem.
% kommen, kam, kme
% fangen, fing, finge
undelayed_get_irreg_v_stem((temo:past_conj),                    (conj:(a_ ConjStem)),  (a_ ConjStem))  if !,prolog(ConjStem\=none).
undelayed_get_irreg_v_stem((temo:past_conj),                    (past:(a_ ConjStem)),  (a_ ConjStem))  if !,prolog(ConjStem=none).


undelayed_get_irreg_v_stem(_,          _,                     (a_ none    ))  if true.


get_irreg_n_stem((num:pl),(a_ UmlautStem),  (a_ UmlautStem))  if !.
get_irreg_n_stem(_,_,_) if true.
